@@ -3,97 +3,45 @@
 """Tests for `acasclient` package."""
 
 import os
-import tempfile
 from datetime import datetime
-import unittest
 import uuid
 import logging
 from pathlib import Path
 
-from acasclient import acasclient
 from acasclient.ddict import ACASDDict, ACASLsThingDDict
 from acasclient.lsthing import (BlobValue, CodeValue, FileValue, LsThingValue,
                                 SimpleLsThing, get_lsKind_to_lsvalue)
 from acasclient.validation import ValidationResult, get_validation_response
+from tests.test_acasclient import BaseAcasClientTest
 
-# SETUP
-# "bob" user name registered
-# "PROJ-00000001" registered
 
 logger = logging.getLogger(__name__)
 logging.basicConfig(level=logging.INFO)
 
 # Constants
-ACAS_DDICT = 'ACAS DDICT'
-ACAS_LSTHING = 'ACAS LsThing'
-PROJECT_METADATA = 'project metadata'
-PROJECT = 'project'
-PROJECT_NAME = 'project name'
-PROJECT_ALIAS = 'project alias'
-STATUS = 'status'
-PROJECT_STATUS = 'project status'
-PROCEDURE_DOCUMENT = 'procedure document'
-PARENT_PROJECT = 'parent project'
-BOOLEAN = 'boolean'
-IS_RESTRICTED = 'is restricted'
-RESTRICTED = 'restricted'
-PDF_DOCUMENT = 'pdf document'
-START_DATE = 'start date'
-NAME_KEY = 'name'
-IS_RESTRICTED_KEY = 'is_restricted'
-STATUS_KEY = 'status'
-START_DATE_KEY = 'start_date'
-DESCRIPTION_KEY = 'description'
-PDF_DOCUMENT_KEY = 'pdf_document'
-PROCEDURE_DOCUMENT_KEY = 'procedure_document'
-PARENT_PROJECT_KEY = 'parent_project'
-ACTIVE = 'active'
-INACTIVE = 'inactive'
+from tests.project_thing import (
+    PROJECT_METADATA, PROJECT, STATUS, PROJECT_STATUS,PROCEDURE_DOCUMENT,PDF_DOCUMENT, 
+    NAME_KEY, IS_RESTRICTED_KEY, STATUS_KEY, START_DATE_KEY, DESCRIPTION_KEY, 
+    PDF_DOCUMENT_KEY, PROCEDURE_DOCUMENT_KEY, PARENT_PROJECT_KEY, ACTIVE, INACTIVE,
+    Project
+)
 
 FWD_ITX = 'relates to'
 BACK_ITX = 'is related to'
 
-
-class Project(SimpleLsThing):
-    ls_type = PROJECT
-    ls_kind = PROJECT
-    preferred_label_kind = PROJECT_NAME
-
-    def __init__(self, name=None, alias=None, start_date=None, description=None, status=None, is_restricted=True, procedure_document=None, pdf_document=None, recorded_by=None,
-                 parent_project=None, ls_thing=None):
-        names = {PROJECT_NAME: name, PROJECT_ALIAS: alias}
-        metadata = {
-            PROJECT_METADATA: {
-                START_DATE: start_date,
-                DESCRIPTION_KEY: description,
-                PROJECT_STATUS: CodeValue(status, PROJECT, STATUS, ACAS_DDICT),
-                IS_RESTRICTED: CodeValue(str(is_restricted).lower(), BOOLEAN, BOOLEAN, ACAS_DDICT),
-                PROCEDURE_DOCUMENT: BlobValue(file_path=procedure_document),
-                PARENT_PROJECT: CodeValue(parent_project, PROJECT, PROJECT, ACAS_LSTHING),
-                PDF_DOCUMENT: FileValue(file_path=pdf_document)
-            }
-        }
-        super(Project, self).__init__(ls_type=self.ls_type, ls_kind=self.ls_kind, names=names, recorded_by=recorded_by,
-                                      preferred_label_kind=self.preferred_label_kind, metadata=metadata, ls_thing=ls_thing)
-
-
-class TestLsThing(unittest.TestCase):
+class TestLsThing(BaseAcasClientTest):
     """Tests for `acasclient lsthing` package model."""
 
-    def setUp(self):
-        """Set up test fixtures, if any."""
-        creds = acasclient.get_default_credentials()
-        self.client = acasclient.client(creds)
-        self.tempdir = tempfile.mkdtemp()
+    # See test_acasclient.BaseAcasClientTest for setUp
 
     def tearDown(self):
         """Tear down test fixtures, if any."""
-        self.client.close()
         files_to_delete = ['dummy.pdf', 'dummy2.pdf']
         for f in files_to_delete:
             file = Path(f)
             if file.exists():
                 os.remove(file)
+        super().tearDown()
 
     # Helpers
     def _get_path(self, file_name):
@@ -118,7 +66,7 @@ class TestLsThing(unittest.TestCase):
         new_bytes = self._get_bytes(file_path)
         orig_bytes = self._get_bytes(orig_file_path)
         self.assertEqual(new_bytes, orig_bytes)
-    
+
     def _test_codevalue_missing_error(self, message, value, code_type, code_kind, code_origin):
         base_msg = "'{code}' is not yet in the database as a valid '{code_kind}'. Please double-check the spelling and correct your data if you expect this to match an existing term. If this is a novel valid term, please contact your administrator to add it to the following dictionary: Code Type: {code_type}, Code Kind: {code_kind}, Code Origin: {code_origin}"
         expected_msg = base_msg.format(code=value, code_type=code_type, code_kind=code_kind, code_origin=code_origin)
@@ -612,7 +560,7 @@ class TestLsThing(unittest.TestCase):
         assert len(proj_1.links) == 1
         # skip CodeValue validation since this is not a valid status
         proj_1.save(self.client, skip_validation=True)
-       
+
         # Run advanced search by interaction w/value matching on the interaction thing
         # Forward interaction query w/interaction thing values
         # Code value search
@@ -725,7 +673,7 @@ class TestLsThing(unittest.TestCase):
                                        max_results=1000,
                                        combine_terms_with_and=True)
         assert len(results) == 0
-    
+
     def test_008_validate_codevalue(self):
         """Test creating CodeValues using code + code_type + code_kind + code_origin
         Confirm that invalid code values are rejected by validate method.
@@ -836,15 +784,18 @@ class TestLsThing(unittest.TestCase):
         assert bad_project_code in validation_response['results']['htmlSummary']
 
 
-class TestBlobValue(unittest.TestCase):
+    def test_008_get_by_code(self):
+        """
+        If no lsthing entry is found for the given `code_name`, `ls_type` and
+        `ls_kind` then `get_by_code` should raise KeyError.
+        """
 
-    def setUp(self) -> None:
-        creds = acasclient.get_default_credentials()
-        self.client = acasclient.client(creds)
+        with self.assertRaises(KeyError):
+            _ = SimpleLsThing.get_by_code(code_name='baz', client=self.client,
+            ls_type='foo', ls_kind='bar')
 
-    def tearDown(self):
-        """Tear down test fixtures, if any."""
-        self.client.close()
+
+class TestBlobValue(BaseAcasClientTest):
 
     def test_as_dict(self):
         """
@@ -860,15 +811,7 @@ class TestBlobValue(unittest.TestCase):
         assert blob_value_dict['comments'] == comments
         assert blob_value_dict['id'] == id
 
-class TestValidationResponse(unittest.TestCase):
-
-    def setUp(self) -> None:
-        creds = acasclient.get_default_credentials()
-        self.client = acasclient.client(creds)
-
-    def tearDown(self):
-        """Tear down test fixtures, if any."""
-        self.client.close()
+class TestValidationResponse(BaseAcasClientTest):
 
     def test_001_response_with_errors_and_warnings(self):
         """
@@ -941,7 +884,7 @@ class TestValidationResponse(unittest.TestCase):
             assert False
         except ValueError as e:
             assert str(e) == "ValidationResult cannot be valid and contain error messages"
-    
+
     def test_002_html_summary(self):
         """
         Test the HTML Summary generated by ValidationResult
